@@ -9,7 +9,7 @@
 # Override the path to the Android NDK, $ANDROID_NDK by default
 # ndk_path = "/path/to/android-ndk"
 # Build tools version to be used
-build_tools = "20.0.0"
+# build_tools = "21.1.2"
 
 # Command for running Ant
 # ant = "/path/to/ant"
@@ -52,6 +52,15 @@ archive_description = """
 The repository of older versions of applications from the main demo repository.
 """
 
+# `fdroid update` will create a link to the current version of a given app.
+# This provides a static path to the current APK.  To disable the creation of
+# this link, uncomment this:
+# make_current_version_link = False
+
+# By default, the "current version" link will be based on the "Name" of the
+# app from the metadata.  You can change it to use a different field from the
+# metadata here:
+# current_version_name_source = 'id'
 
 # The ID of a GPG key for making detached signatures for apks. Optional.
 # gpgkey = '1DBA2E89'
@@ -165,6 +174,16 @@ wiki_password = "1234"
 # machine.
 update_stats = False
 
+# When used with stats, this is a list of IP addresses that are ignored for
+# calculation purposes.
+stats_ignore = []
+
+# Server stats logs are retrieved from. Required when update_stats is True.
+stats_server = "example.com"
+
+# User stats logs are retrieved from. Required when update_stats is True.
+stats_user = "bob"
+
 # Use the following to push stats to a Carbon instance:
 stats_to_carbon = False
 carbon_host = '0.0.0.0'
