@@ -109,14 +109,12 @@ def check_tags(app, pattern):
 
         vcs.gotorevision(None)
 
-        flavour = None
+        flavours = []
         if len(app['builds']) > 0:
             if app['builds'][-1]['subdir']:
                 build_dir = os.path.join(build_dir, app['builds'][-1]['subdir'])
             if app['builds'][-1]['gradle']:
-                flavour = app['builds'][-1]['gradle']
-        if flavour == 'yes':
-            flavour = None
+                flavours = app['builds'][-1]['gradle']
 
         hpak = None
         htag = None
@@ -136,7 +134,7 @@ def check_tags(app, pattern):
             vcs.gotorevision(tag)
 
             # Only process tags where the manifest exists...
-            paths = common.manifest_paths(build_dir, flavour)
+            paths = common.manifest_paths(build_dir, flavours)
             version, vercode, package = \
                 common.parse_androidmanifests(paths, app['Update Check Ignore'])
             if not package or package != appid or not version or not vercode:
@@ -196,20 +194,17 @@ def check_repomanifest(app, branch=None):
         elif repotype == 'bzr':
             vcs.gotorevision(None)
 
-        flavour = None
-
+        flavours = []
         if len(app['builds']) > 0:
             if app['builds'][-1]['subdir']:
                 build_dir = os.path.join(build_dir, app['builds'][-1]['subdir'])
             if app['builds'][-1]['gradle']:
-                flavour = app['builds'][-1]['gradle']
-        if flavour == 'yes':
-            flavour = None
+                flavours = app['builds'][-1]['gradle']
 
         if not os.path.isdir(build_dir):
             return (None, "Subdir '" + app['builds'][-1]['subdir'] + "'is not a valid directory")
 
-        paths = common.manifest_paths(build_dir, flavour)
+        paths = common.manifest_paths(build_dir, flavours)
 
         version, vercode, package = \
             common.parse_androidmanifests(paths, app['Update Check Ignore'])
@@ -319,15 +314,13 @@ def check_changed_subdir(app):
     if not os.path.isdir(build_dir):
         return None
 
-    flavour = None
+    flavours = []
     if len(app['builds']) > 0 and app['builds'][-1]['gradle']:
-        flavour = app['builds'][-1]['gradle']
-    if flavour == 'yes':
-        flavour = None
+        flavours = app['builds'][-1]['gradle']
 
     for d in dirs_with_manifest(build_dir):
         logging.debug("Trying possible dir %s." % d)
-        m_paths = common.manifest_paths(d, flavour)
+        m_paths = common.manifest_paths(d, flavours)
         package = common.parse_androidmanifests(m_paths, app['Update Check Ignore'])[2]
         if package and package == appid:
             logging.debug("Manifest exists in possible dir %s." % d)
@@ -352,18 +345,15 @@ def fetch_autoname(app, tag):
     except VCSException:
         return None
 
-    flavour = None
+    flavours = []
     if len(app['builds']) > 0:
         if app['builds'][-1]['subdir']:
             app_dir = os.path.join(app_dir, app['builds'][-1]['subdir'])
         if app['builds'][-1]['gradle']:
-            flavour = app['builds'][-1]['gradle']
-    if flavour == 'yes':
-        flavour = None
+            flavours = app['builds'][-1]['gradle']
 
-    logging.debug("...fetch auto name from " + app_dir +
-                  ((" (flavour: %s)" % flavour) if flavour else ""))
-    new_name = common.fetch_real_name(app_dir, flavour)
+    logging.debug("...fetch auto name from " + app_dir)
+    new_name = common.fetch_real_name(app_dir, flavours)
     commitmsg = None
     if new_name:
         logging.debug("...got autoname '" + new_name + "'")
@@ -375,7 +365,7 @@ def fetch_autoname(app, tag):
         logging.debug("...couldn't get autoname")
 
     if app['Current Version'].startswith('@string/'):
-        cv = common.version_name(app['Current Version'], app_dir, flavour)
+        cv = common.version_name(app['Current Version'], app_dir, flavours)
         if app['Current Version'] != cv:
             app['Current Version'] = cv
             if not commitmsg:
@@ -570,13 +560,13 @@ def main():
                                      .format(common.getappname(app), version))
         return
 
-    for app in apps:
+    for appid, app in apps.iteritems():
 
         if options.autoonly and app['Auto Update Mode'] in ('None', 'Static'):
-            logging.debug("Nothing to do for {0}...".format(app['id']))
+            logging.debug("Nothing to do for {0}...".format(appid))
             continue
 
-        logging.info("Processing " + app['id'] + '...')
+        logging.info("Processing " + appid + '...')
 
         checkupdates_app(app)
 
